@@ -1,5 +1,6 @@
 import numpy as np
-from scipy.stats import binom, norm, poisson
+import pandas as pd
+from scipy.stats import binom, norm, poisson, beta
 
 class HC(object) :
     """
@@ -11,6 +12,8 @@ class HC(object) :
     [2] Donoho, D. L. and Jin, J. "Higher critcism thresholding: Optimal 
     feature selection when useful features are rare and weak", proceedings
     of the national academy of sciences, 2008.
+    [3] Kipnis, A. "Higher Criticism for Discriminating Word Frequency Tables
+     and Testing Authorship," 2019. 
     ========================================================================
 
     Args:
@@ -84,8 +87,6 @@ class HC(object) :
         -log(BJ) score, P-value attaining it
         """
 
-        from scipy.stats import beta
-
         spv = self._pvals
         N = self._N
 
@@ -238,6 +239,9 @@ def binom_test_two_sided(x, n, p) :
     fron scipy.python.binom_test. It is unclear which one is 
     more accurate.
     """
+
+    n = n.astype(int)
+
     x_low = n * p - np.abs(x-n*p)
     x_high = n * p + np.abs(x-n*p)
 
@@ -260,6 +264,8 @@ def binom_test_two_sided_random(x, n, p) :
     x_low = n * p - np.abs(x-n*p)
     x_high = n * p + np.abs(x-n*p)
 
+    n = n.astype(int)
+
     p_up = binom.cdf(x_low, n, p)\
         + binom.sf(x_high-1, n, p)
     
@@ -298,6 +304,66 @@ def two_sample_test(X, Y, gamma=0.25,
     
     return hc_star, p_thresh
 
+def binom_var_test_df(c1, c2, sym=False, max_m=-1) :
+    """ Binmial variance test along stripes. 
+        This version returns all quantities in the calculation
+    Args:
+    ----
+    c1, c2 : list of integers represents count data from two sample
+    sym : flag indicates wether the size of both sample is assumed
+          identical, hence p=1/2
+    """
+
+    df_smp = pd.DataFrame({'n1' : c1, 'n2' : c2})
+    df_smp.loc[:,'N'] = df_smp.agg('sum', axis = 'columns')
+
+    if max_m > 0 :
+        df_smp = df_smp[df_smp.n1 + df_smp.n2 <= max_m]
+
+    df_hist = df_smp.groupby(['n1', 'n2']).count().reset_index()
+    # truncate where normal approximation is no longer valid
+    df_hist = df_hist[df_hist.N > np.minimum(df_hist.n1 + df_hist.n2, 9)]
+
+    df_hist.loc[:,'m'] = df_hist.n1 + df_hist.n2
+
+    df_hist.loc[:,'N1'] = df_hist.n1 * df_hist.N
+    df_hist.loc[:,'N2'] = df_hist.n2 * df_hist.N
+
+    df_hist.loc[:,'NN1'] = df_hist.N1.sum()
+    df_hist.loc[:,'NN2'] = df_hist.N2.sum()
+
+    df_hist = df_hist.join(df_hist.filter(['m', 'N1', 'N2', 'N']).groupby('m').agg('sum'),
+                           on = 'm', rsuffix='_m')
+
+    df_hist.loc[:,'p'] = (df_hist['NN1'] - df_hist['N1_m'])\
+            / (df_hist['NN1'] + df_hist['NN2'] - df_hist['N1_m'] - df_hist['N2_m'])
+
+    df_hist.loc[:,'s'] = (df_hist.n1 - df_hist.m * df_hist.p) ** 2 * df_hist.N
+    df_hist.loc[:,'Es'] = df_hist.N_m * df_hist.m * df_hist.p * (1 - df_hist.p)
+    df_hist.loc[:,'Vs'] = 2 * df_hist.N_m *  df_hist.m * (df_hist.m) * ( df_hist.p * (1 - df_hist.p) ) ** 2
+    df_hist = df_hist.join(df_hist.groupby('m').agg('sum').s, on = 'm', rsuffix='_m')
+    df_hist.loc[:,'z'] = (df_hist.s_m - df_hist.Es) / np.sqrt(df_hist.Vs)
+    #df_hist.loc[:,'pval'] = df_hist.z.apply(lambda z : norm.cdf(-np.abs(z)))
+    df_hist.loc[:,'pval'] = df_hist.z.apply(lambda z : norm.sf(z))
+
+    # handle the case m=1 seperately
+    n1 = df_hist[(df_hist.n1 == 1) & (df_hist.n2 == 0)].N.values
+    n2 = df_hist[(df_hist.n1 == 0) & (df_hist.n2 == 1)].N.values
+    if len(n1) + len(n2) >= 2 :
+        df_hist.loc[df_hist.m == 1,'pval'] = binom_test_two_sided(n1, n1 + n2 , 1/2)
+
+    return df_hist
+
+def binom_var_test(c1, c2, sym=False, max_m=-1) :
+    """ Binmial variance test along stripes
+    Args:
+    ----
+    c1, c2 : list of integers represents count data from two sample
+    sym : flag indicates wether the size of both sample is assumed
+          identical, hence p=1/2
+    """
+    df_hist = binom_var_test_df(c1, c2, sym=sym, max_m=max_m)
+    return df_hist.groupby('m').pval.mean()
 
 def two_sample_pvals(c1, c2, randomize=False, sym=False):
 
@@ -329,6 +395,7 @@ def two_sample_pvals(c1, c2, randomize=False, sym=False):
 def two_sample_test_df(X, Y, gamma=0.25, min_cnt=0,
                 stbl=True, randomize=False):
     """
+
     Same as two_sample_test but returns all information for computing
     HC score of the two samples as a pandas data DataFrame. 
     Requires pandas.
@@ -377,11 +444,12 @@ def two_sample_test_df(X, Y, gamma=0.25, min_cnt=0,
         counts['n2'],
         randomize=randomize
         )
+
     counts['sign'] = np.sign(counts.n1 - (counts.n1 + counts.n2) * counts.p)
 
-    counts.loc[counts.n1 + counts.n2 <= min_cnt, 'pval'] = np.nan
-    pvals = counts.pval
-    hc = HC(pvals[~pvals.isna()], stbl=stbl)
+    counts.loc[counts.n1 + counts.n2 < min_cnt, 'pval'] = np.nan
+    pvals = counts.pval.values
+    hc = HC(pvals[~np.isnan(pvals)], stbl=stbl)
     #hc_star, p_val_thresh = hc_vals(pvals, gamma=gamma, stbl=stbl)
     hc_star, p_val_thresh = hc.HCstar(gamma=gamma)
 
