@@ -6,12 +6,16 @@ from .TwoSampleHC import binom_test_two_sided
 from .TwoSampleHC import HC
 from .TwoSampleHC import two_sample_test_df
 from .TwoSampleHC import poisson_test_random
+from .TwoSampleHC import binom_var_test
+from .TwoSampleHC import binom_var_test_df
 
 __all__ = [
 	'HC',
     'hc_vals',
     'two_sample_test',
     'two_sample_pvals',
+    'binom_var_test',
+    'binom_var_test_df',
     'binom_test_two_sided_random',
     'binom_test_two_sided',
     'two_sample_test_df',
